/*
    This file is part of the source of talk64 V1.5,
    a PC - DoctorV64 communications program
    Copyright (C) 1997  Andreas Sterbenz (stan@sbox.tu-graz.ac.at)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
*/

#include <unistd.h>
#include <stdio.h>
#include <ncurses/curses.h>

#include "global.h"
#include "parsys.h"

void out1byte(unsigned int port, unsigned char c)
{
  __asm__ volatile ("outb %0,%1"
		    ::"a" ((char) c), "d"((unsigned short) port));
}

unsigned char in1byte(unsigned int port)
{
  char _v;
  __asm__ volatile ("inb %1,%0"
		    :"=a" (_v):"d"((unsigned short) port));

  return _v;
}

int getKbChar(void)
{
  int key = getch();
  if( key == 127 ) {  /* backspace ?! */
    return '\b';
  } else {
    return key;
  }
}

void setInputMode(int echomode)
{
  if( echomode == INPUT_MODE_ECHO ) {
    nocbreak();
    echo();
  } else {
    cbreak();
    noecho();
  }
}

void setIoPerm(unsigned int port, int nr, int val)
{
  if( ioperm(port, nr, val) != 0 ) {
    fflush(stdout);
    fprintf(stderr, "%s: Could not acquire IO privileges for accessing port hardware.\n", progname);
    fprintf(stderr, "Make sure that this program is SUID root!\n");
    exit(1);
  }
}

#define DETECT_MAX_CNT 1000

int detectParPort(unsigned int port)
{
  int i;

  setIoPerm(port, 1, 1);
  out1byte(port, 0xaa);
  for( i=0; i<DETECT_MAX_CNT; i++ ) {
    if( in1byte(port) == 0xaa ) break;
  }
  if( i < DETECT_MAX_CNT ) {
    out1byte(port, 0x55);
    for( i=0; i<DETECT_MAX_CNT; i++ ) {
      if( in1byte(port) == 0x55 ) break;
    }
  }
  setIoPerm(port, 1, 0);
  if( i >= DETECT_MAX_CNT ) return 0;
  return 1;
}

unsigned int getParPort(unsigned int port)
{
  uid_t uid;
  unsigned int parPortAddresses[] = { 0x3bc, 0x378, 0x278 };
  int i;
  
  initscr();
  setInputMode(INPUT_MODE_ECHO);
  if( port == 0 ) port = 1;
  if( port <= 3 ) {
    for( i=0; i<3; i++ ) {
      port -= detectParPort(parPortAddresses[i]);
      if( port == 0 ) {
        port = parPortAddresses[i];
        break;
      }
    }
    if( i >= 3 ) return 0;
  } else {
    if( (port != parPortAddresses[0]) && (port != parPortAddresses[1]) && (port != parPortAddresses[2]) ) {
      fprintf(stderr, "%s: %X is not a valid port address.\n", progname, port);
      fprintf(stderr, "Use the hardware addresses %X, %X, or %X\n", parPortAddresses[0], parPortAddresses[1], parPortAddresses[2]);
      fprintf(stderr, "or specify 1, 2, or 3 for your first, second, or third parallel port.\n");
      exit(1);
    }
  }
  if( port != 0 ) {
    setIoPerm(port, 3, 1);
    uid = getuid();
    seteuid(uid);
  }
  return port;
}
